package com.softwareag.naturalone.natural.natstyle.check.src.parser;

import java.io.Closeable;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;

import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTArrayIndicesNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTBlockStatementEndNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTBlockStatementNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTCommentNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTCommentRootNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTContextNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTDdmNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTDynamicDefinitionNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTGlobalBlockNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTGlobalNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTIdentifierReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTIndependentNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTLabelNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTLocalNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTObjectNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTOperandNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTParameterNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTPositioningNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTReportNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTRootNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTRuleTextNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSessionParameterCombNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSessionParameterNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSessionParameterRootNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSessionParameterValueNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSimpleConstantReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTStatementInputNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTStatementNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTStatementRulevarNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTStatementSetKeyNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTStatementWriteNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSubClauseNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSubroutineNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSymbolConstNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSymbolDDMFieldNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSymbolDDMGroupNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSymbolFieldNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSymbolFillerNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSymbolGroupNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSymbolRedefineNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSymbolViewNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemFunctionLoopReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemFunctionMathReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemFunctionMinMaxValReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemFunctionMiscReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemFunctionReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemFunctionTranslateReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemFunctionTrimReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemVariableForOccReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemVariableReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemVariableSimpleReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemVariableWithFieldReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemVariableWithLabelReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTSystemVariableWithReportReferenceNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTTokenNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTViewNode;
import com.softwareag.naturalone.natural.parser.ast.internal.INaturalASTVisitor;

/**
 * Converts AST into XML. 
 * It drops all nodes with whitespace. 
 * It drops all values but the leaf values.
 */
class XmlVisitor implements INaturalASTVisitor, Closeable {

    private PrintWriter xml;

    public XmlVisitor(String name) throws IOException {
        xml = new PrintWriter(new FileWriter(name));
    }

    @Override
    public void close() {
        xml.flush();
        xml.close();
        xml = null;
    }

    private void log(INaturalASTNode node) {
        boolean hasChildren = node.hasChildren();
        String image = node.getPartition().getImage().trim(); // drop all whitespace
        boolean hasImage = !image.isEmpty();

        if (hasChildren || hasImage) {
            xml.println("<" + node.getNaturalASTNodeType() + ">");

            boolean isLeaf = !hasChildren;
            if (isLeaf && hasImage) {
                String xmlString = image.replaceAll("<", "&lt;");
                xml.println(xmlString);
            }

            // all subclasses of INaturalASTNode behave like that
            // public void accept(INaturalASTVisitor astVisitor) {
            //       if (inaturalastvisitor.visit(this))
            //           acceptChildren(astVisitor);
            // }
            // so it is safe to call it myself for each node.
            node.acceptChildren(this);

            xml.println("</" + node.getNaturalASTNodeType() + ">");
        }
    }

    @Override
    public boolean visit(INaturalASTRootNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTStatementNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTStatementInputNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTStatementRulevarNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTStatementWriteNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTLocalNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTGlobalNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTGlobalBlockNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTParameterNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTContextNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTObjectNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTIndependentNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSymbolConstNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSymbolFieldNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSymbolFillerNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSymbolGroupNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSymbolRedefineNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTCommentNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTCommentRootNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTRuleTextNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSymbolViewNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSymbolDDMFieldNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSymbolDDMGroupNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTBlockStatementNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTBlockStatementEndNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSessionParameterNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSessionParameterRootNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSessionParameterCombNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSessionParameterValueNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTPositioningNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTIdentifierReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSimpleConstantReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTOperandNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTDynamicDefinitionNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTTokenNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTViewNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTDdmNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSubroutineNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSubClauseNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemVariableReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemVariableWithFieldReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemVariableWithLabelReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemVariableWithReportReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemVariableSimpleReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemVariableForOccReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTReportNode node) {
        log((INaturalASTNode) node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTLabelNode node) {
        log((INaturalASTNode) node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemFunctionReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemFunctionLoopReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemFunctionMathReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemFunctionMiscReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemFunctionMinMaxValReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemFunctionTranslateReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTSystemFunctionTrimReferenceNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTStatementSetKeyNode node) {
        log(node);
        return false;
    }

    @Override
    public boolean visit(INaturalASTArrayIndicesNode node) {
        log(node);
        return false;
    }

}
