package org.codecop.natural.natstyle;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.codecop.natural.natstyle.util.IO;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

import net.sourceforge.pmd.Report;
import net.sourceforge.pmd.renderers.XMLRenderer;

public class NatStyleToPmd {

    private final Map<String, Report> reportsByFile = new HashMap<>();

    public void convert(File natStyleXml, File pmdXml) throws ParserConfigurationException, SAXException, IOException {
        parse(natStyleXml);
        report(pmdXml);
    }

    private void parse(File natStyleXml) throws ParserConfigurationException, SAXException, IOException {
        SAXParserFactory spf = SAXParserFactory.newInstance();
        spf.setNamespaceAware(true);
        SAXParser saxParser = spf.newSAXParser();
        XMLReader xmlReader = saxParser.getXMLReader();
        xmlReader.setContentHandler(new NatStyleResultParser(reportsByFile));
        try (Reader reader = new FileReader(natStyleXml)) {
            xmlReader.parse(new InputSource(reader));
        }
    }

    private void report(File pmdXml) throws IOException {
        try (Writer writer = new FileWriter(pmdXml)) {
            XMLRenderer renderer = new XMLRenderer();
            renderer.setWriter(writer);
            renderer.start();
            reportsByFile.values().stream().forEach(IO.uncheck(renderer::renderFileReport));
            renderer.end();
        }
    }

    public static void main(String... args) throws IOException, ParserConfigurationException, SAXException {
        if (args.length < 2) {
            System.out.println("NatStyleToPmd NATstyleResult.xml pmd.xml");
            System.exit(0);
        }

        File natStyleXml = new File(args[0]);
        File pmdXml = new File(args[1]);

        new NatStyleToPmd().convert(natStyleXml, pmdXml);
    }

}
