package org.codecop.natural.natstyle.ant;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.ParserConfigurationException;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.codecop.natural.natstyle.NatStyleToPmd;
import org.xml.sax.SAXException;

/**
 * Ant task wrapper around NatStyleToPmd.
 * 
 * @see NatStyleToPmd
 */
public class NatStyleResultToPmdTask extends Task {

    private File natstyleResult;
    private File pmdResult;

    public void setNatstyleResult(File natstyle) {
        this.natstyleResult = natstyle;
    }

    public void setPmdResult(File pmd) {
        this.pmdResult = pmd;
    }

    @Override
    public void execute() {
        validate();
        logFiles();
        convert();
    }

    private void validate() {
        if (natstyleResult == null) {
            throw new BuildException("natstyleResult not set");
        }

        if (pmdResult == null) {
            throw new BuildException("pmdResult not set");
        }
    }

    private void logFiles() {
        log("Reading NATstyle result from " + natstyleResult, Project.MSG_INFO);
        log("Writing PMD result to " + pmdResult, Project.MSG_INFO);
    }

    private void convert() {
        try {

            new NatStyleToPmd().convert(natstyleResult, pmdResult);

        } catch (ParserConfigurationException | SAXException | IOException e) {
            throw new BuildException("error during XML conversion: " + e.toString(), e);
        }
    }
}
