package org.codecop.natural.natstyle;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.stream.Collectors;
import java.util.stream.StreamSupport;

import javax.xml.parsers.ParserConfigurationException;

import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.junit.runners.Parameterized;
import org.junit.runners.Parameterized.Parameter;
import org.junit.runners.Parameterized.Parameters;
import org.xml.sax.SAXException;
import org.xmlunit.builder.DiffBuilder;
import org.xmlunit.builder.Input;
import org.xmlunit.diff.Diff;

@RunWith(Parameterized.class)
public class NatStyleToPmdTest {

    private static final File FOLDER = new File("./test/org/codecop/natural/natstyle");

    @Parameters(name = "{0}")
    public static Collection<Object[]> testCases() {
        return Arrays.asList( //
                new Object[] { "noViolations" }, //
                new Object[] { "singleViolation" }, //
                new Object[] { "singleOtherViolation" }, //
                new Object[] { "multipleViolations" }, //
                new Object[] { "parserErrors" } //
        );
    }

    @Parameter
    public String tag;

    @Test
    public void shouldConvertViolations() throws ParserConfigurationException, IOException, SAXException {
        convert();
        assertEquals();
    }

    private void convert() throws ParserConfigurationException, IOException, SAXException {
        File input = new File(FOLDER, tag + "_NATstyleResult.xml");
        File output = new File(FOLDER, tag + "_received.xml");
        NatStyleToPmd.main(input.getAbsolutePath().toString(), output.getAbsolutePath().toString());
    }

    private void assertEquals() {
        File actual = new File(FOLDER, tag + "_received.xml");
        File expected = new File(FOLDER, tag + "_pmd.xml");

        Diff diff = DiffBuilder.compare(Input.fromFile(expected)). //
                withTest(Input.fromFile(actual)). //
                ignoreWhitespace(). //
                withAttributeFilter(attr -> !attr.getName().equals("timestamp")). //
                build();

        if (diff.hasDifferences()) {
            String msg = StreamSupport.stream(diff.getDifferences().spliterator(), false). //
                    map(Object::toString). //
                    collect(Collectors.joining("\n"));
            Assert.fail(msg);
        }

        Assert.assertTrue(actual.delete());
    }

}
